@@ -62,7 +62,7 @@ enum dimP {
 #define LOGLIK_SLOT(x) REAL(getListElement(x, "logLik"))
 #define SCOREOBS_SLOT(x) REAL(getListElement(x, "score_obs"))
 #define SCORESBJ_SLOT(x) REAL(getListElement(x, "score_sbj"))
-#define SCORE_SLOT(x) REAL(getListElement(x, "score"))
+#define SCORE_SLOT(x) REAL(getListElement(x, "score"))xf
 #define INFO_SLOT(x) REAL(getListElement(x, "info"))
 #define GHX_SLOT(x) REAL(getListElement(x, "ghx"))
 #define GHW_SLOT(x) REAL(getListElement(x, "ghw"))
@@ -114,14 +114,24 @@ double olmm_GLink(double x, int link) {
  */
 
 SEXP olmm_setPar(SEXP x, SEXP par) {
-  
-  int *dims = DIMS_SLOT(x);
 
-  double *fixef = FIXEF_SLOT(x), /* used slots */
-    *vecRanefCholFac = RANEFCHOLFAC_SLOT(x),
-    *coefficients = COEFFICIENTS_SLOT(x), 
-    *ranefElMat = RANEFELMAT_SLOT(x),
-    *newPar = REAL(par);
+  /* list with the results */
+  SEXP rvalR = PROTECT(allocVector(VECSXP, 4));
+
+  /* R objects which are not modified */
+  int *dims = INTEGER(getListElement(x, "dims"));
+  double *newPar = REAL(par);
+ 
+  /* slots of 'x' to modify */
+  SEXP fixefR = duplicate(getListElement(x, "fixef")),
+    vecRanefCholFacR = duplicate(getListElement(x, "ranefCholFac")),
+    coefficientsR = duplicate(getListElement(x, "coefficients")),
+    ranefElMatR = duplicate(getListElement(x, "ranefElMat"));
+  
+  double *fixef = REAL(fixefR), /* used slots */
+    *vecRanefCholFac = REAL(vecRanefCholFacR),
+    *coefficients = REAL(coefficientsR), 
+    *ranefElMat = REAL(ranefElMatR);
 
   const int p = dims[p_POS], family = dims[fTyp_POS],  
     pCe = dims[pCe_POS], pGe = dims[pGe_POS],
@@ -161,8 +171,15 @@ SEXP olmm_setPar(SEXP x, SEXP par) {
   		  &one, ranefElMat, 
 		  &lenVRanefCholFac, vRanefCholFac, &i1, 
 		  &zero, vecRanefCholFac, &i1);
+
+  /* write updated coefficients into a list */
+  SET_VECTOR_ELT(rvalR, 0, fixefR);
+  SET_VECTOR_ELT(rvalR, 1, vecRanefCholFacR);
+  SET_VECTOR_ELT(rvalR, 2, coefficientsR);
+  SET_VECTOR_ELT(rvalR, 3, ranefElMatR);
   
-  return R_NilValue;
+  UNPROTECT(1);
+  return rvalR;
 }
 
 
@@ -177,11 +194,13 @@ SEXP olmm_setPar(SEXP x, SEXP par) {
  */
 
 SEXP olmm_update_marg(SEXP x, SEXP par) {
-
-  SEXP newPar = PROTECT(duplicate(par));
-
+  
+  /* list with the results (the modified slots) */
+  SEXP rvalR = PROTECT(allocVector(VECSXP, 11));
+  
   /* get subject slot */  
-  int *subject = INTEGER(coerceVector(getListElement(x, "subject"), INTSXP));
+  int *subject =
+    INTEGER(coerceVector(getListElement(x, "subject"), INTSXP));
 
   /* integer valued slots */
   int *dims = DIMS_SLOT(x);
@@ -192,20 +211,29 @@ SEXP olmm_update_marg(SEXP x, SEXP par) {
     *weights_obs = WEIGHTS_SLOT(x),
     *weights_sbj = WEIGHTSSBJ_SLOT(x),
     *offset = OFFSET_SLOT(x), 
-    *eta = ETA_SLOT(x), 
-    *fixef = FIXEF_SLOT(x), 
-    *ranefCholFac = RANEFCHOLFAC_SLOT(x),
-    *logLik_sbj = LOGLIKSBJ_SLOT(x), 
-    *logLik = LOGLIK_SLOT(x), 
-    *score_obs = SCOREOBS_SLOT(x), 
-    *score_sbj = SCORESBJ_SLOT(x), 
-    *score = SCORE_SLOT(x),
-    *info = INFO_SLOT(x),
     *ghw = GHW_SLOT(x), 
-    *ghx = GHX_SLOT(x), 
-    *ranefElMat = RANEFELMAT_SLOT(x);
-  R_CheckStack();
+    *ghx = GHX_SLOT(x);
 
+  /* copy slots to be modified */
+  SEXP etaR = PROTECT(duplicate(getListElement(x, "eta"))),
+    logLik_sbjR = PROTECT(duplicate(getListElement(x, "logLik_sbj"))), 
+    logLikR = PROTECT(duplicate(getListElement(x, "logLik"))), 
+    score_obsR = PROTECT(duplicate(getListElement(x, "score_obs"))), 
+    score_sbjR = PROTECT(duplicate(getListElement(x, "score_sbj"))), 
+    scoreR = PROTECT(duplicate(getListElement(x, "score"))), 
+    infoR = PROTECT(duplicate(getListElement(x, "info")));
+
+  /* create pointers to slots to be modified */
+  double *eta = REAL(etaR),
+    *logLik_sbj = REAL(logLik_sbjR),
+    *logLik = REAL(logLikR),
+    *score_obs = REAL(score_obsR),
+    *score_sbj = REAL(score_sbjR),
+    *score = REAL(scoreR),
+    *info = REAL(infoR);
+    
+  R_CheckStack();
+  
   /* set constants (dimensions of vectors etc.) */
   const int n = dims[n_POS], N = dims[N_POS], 
     p = dims[p_POS], pEta = dims[pEta_POS],
@@ -249,14 +277,18 @@ SEXP olmm_update_marg(SEXP x, SEXP par) {
   R_CheckStack();
 
   if (numGrad == 0) {
-    scoreCond_obs = Calloc((1 - numGrad) * n * nPar + numGrad, double);
-    scoreCond_sbj = Calloc((1 - numGrad) * N * nPar + numGrad, double);
+    scoreCond_obs = Calloc(n * nPar, double);
+    scoreCond_sbj = Calloc(N * nPar, double);
   }
 
   AllocVal(etaRanef, n * nEta, zero);
   
   /* update parameters */
-  olmm_setPar(x, newPar);
+  SEXP newParList = PROTECT(olmm_setPar(x, par));
+  
+  double *fixef = REAL(VECTOR_ELT(newParList, 0)),
+    *ranefCholFac = REAL(VECTOR_ELT(newParList, 1)),
+    *ranefElMat = REAL(VECTOR_ELT(newParList, 3));
   
   /* initialize eta = offset */
   for (int i = 0; i < (n * nEta); i++) eta[i] = offset[i];
@@ -265,7 +297,7 @@ SEXP olmm_update_marg(SEXP x, SEXP par) {
   if (pEta > 0L)
     F77_CALL(dgemm)("N", "N", &n, &nEta, &pEta, &one, X, &n, fixef, 
 		    &pEta, &one, eta, &n);
-
+  
   switch (family) {
   case 1: 
     etaCLM = Calloc(n * 2 , double);
@@ -295,7 +327,7 @@ SEXP olmm_update_marg(SEXP x, SEXP par) {
   }
 
   /* Gauss-Hermite quadrature */
-
+  
   for (int k = 0; k < nQP; k++) {
     
     /* clear temporary objects */
@@ -303,14 +335,13 @@ SEXP olmm_update_marg(SEXP x, SEXP par) {
     AllocVal(logLikCond_sbj, N, zero);
 
     if (numGrad == 0) {
-
-      AllocVal(scoreCond_obs, (1 - numGrad) * n * nPar + numGrad, zero);
-      AllocVal(scoreCond_sbj, (1 - numGrad) * N * nPar + numGrad, zero);
+      AllocVal(scoreCond_obs, n * nPar, zero);
+      AllocVal(scoreCond_sbj, N * nPar, zero);
     }
 
     if ((family == 2) | (family == 3)) AllocVal(sumBL, n, zero);
 
-    /* prepare integration nodes and weights */    
+    /* prepare integration nodes and weights */
     for (int i = 0; i < q; i++) {
       gq_nodes[i] = ghx[nQP * i + k];
       gq_weight *= ghw[nQP * i + k];
@@ -353,7 +384,7 @@ SEXP olmm_update_marg(SEXP x, SEXP par) {
     }
 
     /* approximate marginal log Likelihood ................. */
-    
+
     for (int i = 0; i < n; i++) {     
       
       switch (family) {
@@ -380,7 +411,7 @@ SEXP olmm_update_marg(SEXP x, SEXP par) {
     }
     
     if (numGrad == 0) {
-	
+      
       /* approximate score function ........................ */
       
       for (int i = 0; i < n; i++) {
@@ -610,14 +641,42 @@ SEXP olmm_update_marg(SEXP x, SEXP par) {
   }
 
   /* finish Likelihood function calculations ..... */
-
+  
   for (int i = 0; i < N; i++) {
     logLik_sbj[i] = weights_sbj[i] * log(logLik_sbj[i]);
     logLik[0] += logLik_sbj[i];
   }
 
-  UNPROTECT(1);
-  return R_NilValue;
+  /* write updated coefficients into a list */
+  SET_VECTOR_ELT(rvalR, 0, VECTOR_ELT(newParList, 0));
+  SET_VECTOR_ELT(rvalR, 1, VECTOR_ELT(newParList, 1));
+  SET_VECTOR_ELT(rvalR, 2, VECTOR_ELT(newParList, 2));
+  SET_VECTOR_ELT(rvalR, 3, VECTOR_ELT(newParList, 3));
+  SET_VECTOR_ELT(rvalR, 4, etaR);
+  SET_VECTOR_ELT(rvalR, 5, logLik_sbjR);
+  SET_VECTOR_ELT(rvalR, 6, logLikR);
+  SET_VECTOR_ELT(rvalR, 7, score_obsR);
+  SET_VECTOR_ELT(rvalR, 8, score_sbjR);
+  SET_VECTOR_ELT(rvalR, 9, scoreR);
+  SET_VECTOR_ELT(rvalR, 10, infoR);
+  
+  /* set names of list */
+  SEXP names = PROTECT(allocVector(STRSXP, 11));
+  SET_STRING_ELT(names, 0, mkChar("fixef"));
+  SET_STRING_ELT(names, 1, mkChar("ranefCholFac"));
+  SET_STRING_ELT(names, 2, mkChar("coefficients"));
+  SET_STRING_ELT(names, 3, mkChar("ranefElMat"));
+  SET_STRING_ELT(names, 4, mkChar("eta"));
+  SET_STRING_ELT(names, 5, mkChar("logLik_sbj"));
+  SET_STRING_ELT(names, 6, mkChar("logLik"));
+  SET_STRING_ELT(names, 7, mkChar("score_obs"));
+  SET_STRING_ELT(names, 8, mkChar("score_sbj"));
+  SET_STRING_ELT(names, 9, mkChar("score"));
+  SET_STRING_ELT(names, 10, mkChar("info"));
+  setAttrib(rvalR, R_NamesSymbol, names);
+  
+  UNPROTECT(10);
+  return rvalR;
 }
 
 /**
@@ -631,7 +690,7 @@ SEXP olmm_update_marg(SEXP x, SEXP par) {
  */
 
 SEXP olmm_update_u(SEXP x) {
-
+  
   /* get subject slot */  
   int *subject = INTEGER(coerceVector(getListElement(x, "subject"), INTSXP));
 
@@ -639,11 +698,18 @@ SEXP olmm_update_u(SEXP x) {
   int *dims = DIMS_SLOT(x);
 
   /* numeric valued objects */
-  double *W = W_SLOT(x), *u = U_SLOT(x),
+  double *W = W_SLOT(x), 
     *eta = ETA_SLOT(x), 
     *ranefCholFac = RANEFCHOLFAC_SLOT(x), 
     *logLik_sbj = LOGLIKSBJ_SLOT(x), 
     *ghw = GHW_SLOT(x), *ghx = GHX_SLOT(x);
+
+  /* copy random effect slot to be modified */
+  SEXP uR = PROTECT(duplicate(getListElement(x, "u")));
+
+  /* create a pointer to copy of random effect slot */
+  double *u = REAL(uR);
+    
   R_CheckStack();
 
   /* set constants (dimensions of vectors etc.) */
@@ -795,7 +861,9 @@ SEXP olmm_update_u(SEXP x) {
   if (family == 1) Free(etaRanefCLM);
   if ((family == 2) | (family == 3)) Free(sumBL);
 
-  return R_NilValue;
+  UNPROTECT(1);
+  
+  return uR;
 }
 
 
@@ -816,8 +884,11 @@ SEXP olmm_update_u(SEXP x) {
 
 SEXP olmm_pred_marg(SEXP x, SEXP eta, SEXP W, SEXP n, SEXP pred) {
 
-  double *reta = REAL(eta), *rW = REAL(W), *rpred = REAL(pred);
-
+  /* duplicate input matrix */
+  SEXP rvalR = PROTECT(duplicate(pred));
+  double *rval = REAL(rvalR);
+  
+  double *reta = REAL(eta), *rW = REAL(W);
   const int rn = INTEGER(n)[0];
 
   /* integer valued slots and pointer to factor valued slots */
@@ -848,7 +919,7 @@ SEXP olmm_pred_marg(SEXP x, SEXP eta, SEXP W, SEXP n, SEXP pred) {
     *ranef = Alloca(qEta * nEta, double);
   R_CheckStack();
   
-  AllocVal(rpred, rn * J, zero);
+  AllocVal(rval, rn * J, zero);
 
   for (int k = 0; k < nQP; k++) {
 
@@ -910,23 +981,27 @@ SEXP olmm_pred_marg(SEXP x, SEXP eta, SEXP W, SEXP n, SEXP pred) {
     
     for (int i = 0; i < rn; i++) {
       for (int j = 0; j < J; j++) {
-	rpred[rn * j + i] += exp(predCond[rn * j + i] + gq_weight);
+	rval[rn * j + i] += exp(predCond[rn * j + i] + gq_weight);
       }
     }
     
     gq_weight = 1; /* reset integration weight */
   }
   Free(etaRanef);
-  Free(predCond);
-  return R_NilValue;
+  Free(predCond);  
+  UNPROTECT(1); 
+  return rvalR;
 }
 
 
 SEXP olmm_pred_margNew(SEXP x, SEXP etaNew, SEXP WNew, SEXP subjectNew, 
 		       SEXP nNew, SEXP pred) {
+
+  /* duplicate input matrix */
+  SEXP rvalR = PROTECT(duplicate(pred));
+  double *rval = REAL(rvalR);
   
-  double *retaNew = REAL(etaNew), *rWNew = REAL(WNew), 
-    *rpred = REAL(pred);
+  double *retaNew = REAL(etaNew), *rWNew = REAL(WNew);
   int *rsubjectNew = INTEGER(subjectNew);
   const int rnNew = INTEGER(nNew)[0];
   
@@ -970,7 +1045,7 @@ SEXP olmm_pred_margNew(SEXP x, SEXP etaNew, SEXP WNew, SEXP subjectNew,
     *ranef = Alloca(qEta * nEta, double);
   R_CheckStack();
   
-  AllocVal(rpred, rnNew * J, zero);
+  AllocVal(rval, rnNew * J, zero);
 
   for (int k = 0; k < nQP; k++) {
     
@@ -1072,7 +1147,7 @@ SEXP olmm_pred_margNew(SEXP x, SEXP etaNew, SEXP WNew, SEXP subjectNew,
     for (int i = 0; i < rnNew; i++) {
       for (int j = 0; j < J; j++) {
 	lTmp = rsubjectNew[i] < N ? logLik_sbj[rsubjectNew[i] - 1] : 0;
-	rpred[rnNew * j + i] += 
+	rval[rnNew * j + i] += 
 	  exp(predCond[rnNew * j + i] + gq_weight + logLikCond_sbj[i] - lTmp);
       }
     }
@@ -1085,5 +1160,6 @@ SEXP olmm_pred_margNew(SEXP x, SEXP etaNew, SEXP WNew, SEXP subjectNew,
   Free(etaRanef);
   Free(etaRanefNew);
   Free(predCond);
-  return R_NilValue;
+  UNPROTECT(1); 
+  return rvalR;
 }
