@@ -29,8 +29,7 @@ SEXP vcrpart_duplicate(SEXP x) {
  */
 
 SEXP getListElement(SEXP list, const char *str) {
-  SEXP elmt = R_NilValue, 
-    names = getAttrib(list, R_NamesSymbol); 
+  SEXP elmt = R_NilValue, names = getAttrib(list, R_NamesSymbol); 
   for (int i = 0; i < length(list); i++)
     if(strcmp(CHAR(STRING_ELT(names, i)), str) == 0) {
       elmt = VECTOR_ELT(list, i);
